@@ -1,17 +1,15 @@
 """Convert between frames and higher-level AMQP methods."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
-from __future__ import absolute_import, unicode_literals
 
 from collections import defaultdict
+from struct import pack, pack_into, unpack_from
 
 from . import spec
 from .basic_message import Message
 from .exceptions import UnexpectedFrame
-from .five import range, text_t
-from .platform import pack, pack_into, unpack_from
 from .utils import str_to_bytes
 
-__all__ = ['frame_handler', 'frame_writer']
+__all__ = ('frame_handler', 'frame_writer')
 
 #: Set of methods that require both a content frame and a body frame.
 _CONTENT_METHODS = frozenset([
@@ -38,7 +36,7 @@ def frame_handler(connection, callback,
         connection.bytes_recv += 1
         if frame_type not in (expected_types[channel], 8):
             raise UnexpectedFrame(
-                'Received frame {0} while expecting type: {1}'.format(
+                'Received frame {} while expecting type: {}'.format(
                     frame_type, expected_types[channel]),
             )
         elif frame_type == 1:
@@ -85,37 +83,50 @@ def frame_handler(connection, callback,
     return on_frame
 
 
+class Buffer:
+    def __init__(self, buf):
+        self.buf = buf
+
+    @property
+    def buf(self):
+        return self._buf
+
+    @buf.setter
+    def buf(self, buf):
+        self._buf = buf
+        self.view = memoryview(buf)
+
+
 def frame_writer(connection, transport,
                  pack=pack, pack_into=pack_into, range=range, len=len,
-                 bytes=bytes, str_to_bytes=str_to_bytes, text_t=text_t):
+                 bytes=bytes, str_to_bytes=str_to_bytes, text_t=str):
     """Create closure that writes frames."""
     write = transport.write
 
-    # memoryview first supported in Python 2.7
-    # Initial support was very shaky, so could be we have to
-    # check for a bugfix release.
-    buf = bytearray(connection.frame_max - 8)
-    view = memoryview(buf)
+    buffer_store = Buffer(bytearray(connection.frame_max - 8))
 
     def write_frame(type_, channel, method_sig, args, content):
         chunk_size = connection.frame_max - 8
+        # frame_max can be updated via connection._on_tune. If
+        # it became larger, then we need to resize the buffer
+        # to prevent overflow.
+        if chunk_size > len(buffer_store.buf):
+            buffer_store.buf = bytearray(chunk_size)
+        buf = buffer_store.buf
+        view = buffer_store.view
         offset = 0
         properties = None
         args = str_to_bytes(args)
         if content:
             body = content.body
-            if isinstance(body, text_t):
+            if isinstance(body, str):
                 encoding = content.properties.setdefault(
                     'content_encoding', 'utf-8')
                 body = body.encode(encoding)
             properties = content._serialize_properties()
             bodylen = len(body)
-            framelen = (
-                len(args) +
-                (len(properties) or 0) +
-                bodylen +
-                FRAME_OVERHEAD
-            )
+            properties_len = len(properties) or 0
+            framelen = len(args) + properties_len + bodylen + FRAME_OVERHEAD
             bigbody = framelen > chunk_size
         else:
             body, bodylen, bigbody = None, 0, 0
