@@ -1,19 +1,18 @@
 """Transport implementation."""
 # Copyright (C) 2009 Barry Pederson <bp@barryp.org>
-from __future__ import absolute_import, unicode_literals
 
 import errno
+import os
 import re
 import socket
 import ssl
-from ssl import SSLError
 from contextlib import contextmanager
+from ssl import SSLError
+from struct import pack, unpack
 
 from .exceptions import UnexpectedFrame
-from .five import items
-from .platform import KNOWN_TCP_OPTS, SOL_TCP, pack, unpack
-from .utils import get_errno, set_cloexec
-
+from .platform import KNOWN_TCP_OPTS, SOL_TCP
+from .utils import set_cloexec
 
 _UNAVAIL = {errno.EAGAIN, errno.EINTR, errno.ENOENT, errno.EWOULDBLOCK}
 
@@ -24,7 +23,7 @@ EMPTY_BUFFER = bytes()
 SIGNED_INT_MAX = 0x7FFFFFFF
 
 # Yes, Advanced Message Queuing Protocol Protocol is redundant
-AMQP_PROTOCOL_HEADER = 'AMQP\x00\x00\x09\x01'.encode('latin_1')
+AMQP_PROTOCOL_HEADER = b'AMQP\x00\x00\x09\x01'
 
 # Match things like: [fe80::1]:5432, from RFC 2732
 IPV6_LITERAL = re.compile(r'\[([\.0-9a-f:]+)\](?::(\d+))?')
@@ -53,8 +52,37 @@ def to_host_port(host, default=AMQP_PORT):
     return host, port
 
 
-class _AbstractTransport(object):
-    """Common superclass for TCP and SSL transports."""
+class _AbstractTransport:
+    """Common superclass for TCP and SSL transports.
+
+    PARAMETERS:
+        host: str
+
+            Broker address in format ``HOSTNAME:PORT``.
+
+        connect_timeout: int
+
+            Timeout of creating new connection.
+
+        read_timeout: int
+
+            sets ``SO_RCVTIMEO`` parameter of socket.
+
+        write_timeout: int
+
+            sets ``SO_SNDTIMEO`` parameter of socket.
+
+        socket_settings: dict
+
+            dictionary containing `optname` and ``optval`` passed to
+            ``setsockopt(2)``.
+
+        raise_on_initial_eintr: bool
+
+            when True, ``socket.timeout`` is raised
+            when exception is received during first read. See ``_read()`` for
+            details.
+    """
 
     def __init__(self, host, connect_timeout=None,
                  read_timeout=None, write_timeout=None,
@@ -69,6 +97,14 @@ class _AbstractTransport(object):
         self.write_timeout = write_timeout
         self.socket_settings = socket_settings
 
+    def __repr__(self):
+        if self.sock:
+            src = f'{self.sock.getsockname()[0]}:{self.sock.getsockname()[1]}'
+            dst = f'{self.sock.getpeername()[0]}:{self.sock.getpeername()[1]}'
+            return f'<{type(self).__name__}: {src} -> {dst} at {id(self):#x}>'
+        else:
+            return f'<{type(self).__name__}: (disconnected) at {id(self):#x}>'
+
     def connect(self):
         try:
             # are we already connected?
@@ -82,7 +118,7 @@ class _AbstractTransport(object):
             # EINTR, EAGAIN, EWOULDBLOCK would signal that the banner
             # has _not_ been sent
             self.connected = True
-        except (OSError, IOError, SSLError):
+        except (OSError, SSLError):
             # if not fully connected, close socket, and reraise error
             if self.sock and not self.connected:
                 self.sock.close()
@@ -108,8 +144,8 @@ class _AbstractTransport(object):
                     # Non-blocking SSL sockets can throw SSLError
                     raise socket.timeout()
                 raise
-            except socket.error as exc:
-                if get_errno(exc) == errno.EWOULDBLOCK:
+            except OSError as exc:
+                if exc.errno == errno.EWOULDBLOCK:
                     raise socket.timeout()
                 raise
             finally:
@@ -159,7 +195,7 @@ class _AbstractTransport(object):
                         pass
                     self.sock.settimeout(timeout)
                     self.sock.connect(sa)
-                except socket.error as ex:
+                except OSError as ex:
                     e = ex
                     if self.sock is not None:
                         self.sock.close()
@@ -215,7 +251,7 @@ class _AbstractTransport(object):
         tcp_opts = self._get_tcp_socket_defaults(self.sock)
         if socket_settings:
             tcp_opts.update(socket_settings)
-        for opt, val in items(tcp_opts):
+        for opt, val in tcp_opts.items():
             self.sock.setsockopt(SOL_TCP, opt, val)
 
     def _read(self, n, initial=False):
@@ -246,6 +282,17 @@ class _AbstractTransport(object):
         self.connected = False
 
     def read_frame(self, unpack=unpack):
+        """Parse AMQP frame.
+
+        Frame has following format::
+
+            0      1         3         7                   size+7      size+8
+            +------+---------+---------+   +-------------+   +-----------+
+            | type | channel |  size   |   |   payload   |   | frame-end |
+            +------+---------+---------+   +-------------+   +-----------+
+             octet    short     long        'size' octets        octet
+
+        """
         read = self._read
         read_frame_buffer = EMPTY_BUFFER
         try:
@@ -256,47 +303,100 @@ class _AbstractTransport(object):
             # so we know the size can be at most 2 * SIGNED_INT_MAX
             if size > SIGNED_INT_MAX:
                 part1 = read(SIGNED_INT_MAX)
-                part2 = read(size - SIGNED_INT_MAX)
+
+                try:
+                    part2 = read(size - SIGNED_INT_MAX)
+                except (socket.timeout, OSError, SSLError):
+                    # In case this read times out, we need to make sure to not
+                    # lose part1 when we retry the read
+                    read_frame_buffer += part1
+                    raise
+
                 payload = b''.join([part1, part2])
             else:
                 payload = read(size)
             read_frame_buffer += payload
-            ch = ord(read(1))
+            frame_end = ord(read(1))
         except socket.timeout:
             self._read_buffer = read_frame_buffer + self._read_buffer
             raise
-        except (OSError, IOError, SSLError, socket.error) as exc:
-            # Don't disconnect for ssl read time outs
-            # http://bugs.python.org/issue10272
-            if isinstance(exc, SSLError) and 'timed out' in str(exc):
+        except (OSError, SSLError) as exc:
+            if (
+                isinstance(exc, socket.error) and os.name == 'nt'
+                and exc.errno == errno.EWOULDBLOCK  # noqa
+            ):
+                # On windows we can get a read timeout with a winsock error
+                # code instead of a proper socket.timeout() error, see
+                # https://github.com/celery/py-amqp/issues/320
+                self._read_buffer = read_frame_buffer + self._read_buffer
                 raise socket.timeout()
-            if get_errno(exc) not in _UNAVAIL:
+
+            if isinstance(exc, SSLError) and 'timed out' in str(exc):
+                # Don't disconnect for ssl read time outs
+                # http://bugs.python.org/issue10272
+                self._read_buffer = read_frame_buffer + self._read_buffer
+                raise socket.timeout()
+
+            if exc.errno not in _UNAVAIL:
                 self.connected = False
             raise
-        if ch == 206:  # '\xce'
+        # frame-end octet must contain '\xce' value
+        if frame_end == 206:
             return frame_type, channel, payload
         else:
             raise UnexpectedFrame(
-                'Received {0:#04x} while expecting 0xce'.format(ch))
+                f'Received frame_end {frame_end:#04x} while expecting 0xce')
 
     def write(self, s):
         try:
             self._write(s)
         except socket.timeout:
             raise
-        except (OSError, IOError, socket.error) as exc:
-            if get_errno(exc) not in _UNAVAIL:
+        except OSError as exc:
+            if exc.errno not in _UNAVAIL:
                 self.connected = False
             raise
 
 
 class SSLTransport(_AbstractTransport):
-    """Transport that works over SSL."""
+    """Transport that works over SSL.
+
+    PARAMETERS:
+        host: str
+
+            Broker address in format ``HOSTNAME:PORT``.
+
+        connect_timeout: int
+
+            Timeout of creating new connection.
+
+        ssl: bool|dict
+
+            parameters of TLS subsystem.
+                - when ``ssl`` is not dictionary, defaults of TLS are used
+                - otherwise:
+                    - if ``ssl`` dictionary contains ``context`` key,
+                      :attr:`~SSLTransport._wrap_context` is used for wrapping
+                      socket. ``context`` is a dictionary passed to
+                      :attr:`~SSLTransport._wrap_context` as context parameter.
+                      All others items from ``ssl`` argument are passed as
+                      ``sslopts``.
+                    - if ``ssl`` dictionary does not contain ``context`` key,
+                      :attr:`~SSLTransport._wrap_socket_sni` is used for
+                      wrapping socket. All items in ``ssl`` argument are
+                      passed to :attr:`~SSLTransport._wrap_socket_sni` as
+                      parameters.
+
+        kwargs:
+
+            additional arguments of
+            :class:`~amqp.transport._AbstractTransport` class
+    """
 
     def __init__(self, host, connect_timeout=None, ssl=None, **kwargs):
         self.sslopts = ssl if isinstance(ssl, dict) else {}
         self._read_buffer = EMPTY_BUFFER
-        super(SSLTransport, self).__init__(
+        super().__init__(
             host, connect_timeout=connect_timeout, **kwargs)
 
     def _setup_transport(self):
@@ -311,57 +411,131 @@ class SSLTransport(_AbstractTransport):
         return self._wrap_socket_sni(sock, **sslopts)
 
     def _wrap_context(self, sock, sslopts, check_hostname=None, **ctx_options):
+        """Wrap socket without SNI headers.
+
+        PARAMETERS:
+            sock: socket.socket
+
+            Socket to be wrapped.
+
+            sslopts: dict
+
+                Parameters of  :attr:`ssl.SSLContext.wrap_socket`.
+
+            check_hostname
+
+                Whether to match the peer cert’s hostname. See
+                :attr:`ssl.SSLContext.check_hostname` for details.
+
+            ctx_options
+
+                Parameters of :attr:`ssl.create_default_context`.
+        """
         ctx = ssl.create_default_context(**ctx_options)
         ctx.check_hostname = check_hostname
         return ctx.wrap_socket(sock, **sslopts)
 
     def _wrap_socket_sni(self, sock, keyfile=None, certfile=None,
-                         server_side=False, cert_reqs=ssl.CERT_NONE,
+                         server_side=False, cert_reqs=None,
                          ca_certs=None, do_handshake_on_connect=False,
                          suppress_ragged_eofs=True, server_hostname=None,
                          ciphers=None, ssl_version=None):
         """Socket wrap with SNI headers.
 
-        Default `ssl.wrap_socket` method augmented with support for
-        setting the server_hostname field required for SNI hostname header
-        """
-        # Setup the right SSL version; default to optimal versions across
-        # ssl implementations
-        if ssl_version is None:
-            # older versions of python 2.7 and python 2.6 do not have the
-            # ssl.PROTOCOL_TLS defined the equivalent is ssl.PROTOCOL_SSLv23
-            # we default to PROTOCOL_TLS and fallback to PROTOCOL_SSLv23
-            # TODO: Drop this once we drop Python 2.7 support
-            if hasattr(ssl, 'PROTOCOL_TLS'):
-                ssl_version = ssl.PROTOCOL_TLS
-            else:
-                ssl_version = ssl.PROTOCOL_SSLv23
+        stdlib :attr:`ssl.SSLContext.wrap_socket` method augmented with support
+        for setting the server_hostname field required for SNI hostname header.
 
+        PARAMETERS:
+            sock: socket.socket
+
+                Socket to be wrapped.
+
+            keyfile: str
+
+                Path to the private key
+
+            certfile: str
+
+                Path to the certificate
+
+            server_side: bool
+
+                Identifies whether server-side or client-side
+                behavior is desired from this socket. See
+                :attr:`~ssl.SSLContext.wrap_socket` for details.
+
+            cert_reqs: ssl.VerifyMode
+
+                When set to other than :attr:`ssl.CERT_NONE`, peers certificate
+                is checked. Possible values are :attr:`ssl.CERT_NONE`,
+                :attr:`ssl.CERT_OPTIONAL` and :attr:`ssl.CERT_REQUIRED`.
+
+            ca_certs: str
+
+                Path to “certification authority” (CA) certificates
+                used to validate other peers’ certificates when ``cert_reqs``
+                is other than :attr:`ssl.CERT_NONE`.
+
+            do_handshake_on_connect: bool
+
+                Specifies whether to do the SSL
+                handshake automatically. See
+                :attr:`~ssl.SSLContext.wrap_socket` for details.
+
+            suppress_ragged_eofs (bool):
+
+                See :attr:`~ssl.SSLContext.wrap_socket` for details.
+
+            server_hostname: str
+
+                Specifies the hostname of the service which
+                we are connecting to. See :attr:`~ssl.SSLContext.wrap_socket`
+                for details.
+
+            ciphers: str
+
+                Available ciphers for sockets created with this
+                context. See :attr:`ssl.SSLContext.set_ciphers`
+
+            ssl_version:
+
+                Protocol of the SSL Context. The value is one of
+                ``ssl.PROTOCOL_*`` constants.
+        """
         opts = {
             'sock': sock,
-            'keyfile': keyfile,
-            'certfile': certfile,
             'server_side': server_side,
-            'cert_reqs': cert_reqs,
-            'ca_certs': ca_certs,
             'do_handshake_on_connect': do_handshake_on_connect,
             'suppress_ragged_eofs': suppress_ragged_eofs,
-            'ciphers': ciphers,
-            'ssl_version': ssl_version
+            'server_hostname': server_hostname,
         }
 
-        sock = ssl.wrap_socket(**opts)
-        # Set SNI headers if supported
-        if (server_hostname is not None) and (
-                hasattr(ssl, 'HAS_SNI') and ssl.HAS_SNI) and (
-                hasattr(ssl, 'SSLContext')):
-            context = ssl.SSLContext(opts['ssl_version'])
+        if ssl_version is None:
+            ssl_version = (
+                ssl.PROTOCOL_TLS_SERVER
+                if server_side
+                else ssl.PROTOCOL_TLS_CLIENT
+            )
+
+        context = ssl.SSLContext(ssl_version)
+
+        if certfile is not None:
+            context.load_cert_chain(certfile, keyfile)
+        if ca_certs is not None:
+            context.load_verify_locations(ca_certs)
+        if ciphers is not None:
+            context.set_ciphers(ciphers)
+        if cert_reqs is not None:
             context.verify_mode = cert_reqs
-            if cert_reqs != ssl.CERT_NONE:
-                context.check_hostname = True
-            if (certfile is not None) and (keyfile is not None):
-                context.load_cert_chain(certfile, keyfile)
-            sock = context.wrap_socket(sock, server_hostname=server_hostname)
+        # Set SNI headers if supported
+        try:
+            context.check_hostname = (
+                ssl.HAS_SNI and server_hostname is not None
+            )
+        except AttributeError:
+            pass  # ask forgiveness not permission
+
+        sock = context.wrap_socket(**opts)
         return sock
 
     def _shutdown_transport(self):
@@ -380,11 +554,7 @@ class SSLTransport(_AbstractTransport):
             while len(rbuf) < n:
                 try:
                     s = recv(n - len(rbuf))  # see note above
-                except socket.error as exc:
-                    # ssl.sock.read may cause a SSLerror without errno
-                    # http://bugs.python.org/issue10272
-                    if isinstance(exc, SSLError) and 'timed out' in str(exc):
-                        raise socket.timeout()
+                except OSError as exc:
                     # ssl.sock.read may cause ENOENT if the
                     # operation couldn't be performed (Issue celery#1414).
                     if exc.errno in _errnos:
@@ -393,7 +563,7 @@ class SSLTransport(_AbstractTransport):
                         continue
                     raise
                 if not s:
-                    raise IOError('Server unexpectedly closed connection')
+                    raise OSError('Server unexpectedly closed connection')
                 rbuf += s
         except:  # noqa
             self._read_buffer = rbuf
@@ -414,12 +584,15 @@ class SSLTransport(_AbstractTransport):
                 # None.
                 n = 0
             if not n:
-                raise IOError('Socket closed')
+                raise OSError('Socket closed')
             s = s[n:]
 
 
 class TCPTransport(_AbstractTransport):
-    """Transport that deals directly with TCP socket."""
+    """Transport that deals directly with TCP socket.
+
+    All parameters are :class:`~amqp.transport._AbstractTransport` class.
+    """
 
     def _setup_transport(self):
         # Setup to _write() directly to the socket, and
@@ -436,14 +609,14 @@ class TCPTransport(_AbstractTransport):
             while len(rbuf) < n:
                 try:
                     s = recv(n - len(rbuf))
-                except socket.error as exc:
+                except OSError as exc:
                     if exc.errno in _errnos:
                         if initial and self.raise_on_initial_eintr:
                             raise socket.timeout()
                         continue
                     raise
                 if not s:
-                    raise IOError('Server unexpectedly closed connection')
+                    raise OSError('Server unexpectedly closed connection')
                 rbuf += s
         except:  # noqa
             self._read_buffer = rbuf
@@ -457,7 +630,29 @@ def Transport(host, connect_timeout=None, ssl=False, **kwargs):
     """Create transport.
 
     Given a few parameters from the Connection constructor,
-    select and create a subclass of _AbstractTransport.
+    select and create a subclass of
+    :class:`~amqp.transport._AbstractTransport`.
+
+    PARAMETERS:
+
+        host: str
+
+            Broker address in format ``HOSTNAME:PORT``.
+
+        connect_timeout: int
+
+            Timeout of creating new connection.
+
+        ssl: bool|dict
+
+            If set, :class:`~amqp.transport.SSLTransport` is used
+            and ``ssl`` parameter is passed to it. Otherwise
+            :class:`~amqp.transport.TCPTransport` is used.
+
+        kwargs:
+
+            additional arguments of :class:`~amqp.transport._AbstractTransport`
+            class
     """
     transport = SSLTransport if ssl else TCPTransport
     return transport(host, connect_timeout=connect_timeout, ssl=ssl, **kwargs)
