@@ -1,6 +1,6 @@
-from __future__ import absolute_import, unicode_literals
-
 import socket
+from struct import pack
+from unittest.mock import ANY, Mock, call, patch
 
 import pytest
 
@@ -8,10 +8,8 @@ import amqp
 from amqp import Channel, Connection, Message, sasl, spec
 from amqp.exceptions import (AccessRefused, ConnectionError, InvalidCommand,
                              NotFound, PreconditionFailed, ResourceLocked)
-from amqp.platform import pack
 from amqp.protocol import queue_declare_ok_t
 from amqp.serialization import dumps, loads
-from case import ANY, Mock, call, patch
 
 connection_testdata = (
     (spec.Connection.Blocked, '_on_blocked'),
@@ -114,7 +112,7 @@ def build_frame_type_3(channel, body):
     return 3, channel, body
 
 
-class DataComparator(object):
+class DataComparator:
     # Comparator used for asserting serialized data. It can be used
     # in cases when direct comparision of bytestream cannot be used
     # (mainly cases of Table type where order of items can vary)
@@ -123,7 +121,7 @@ class DataComparator(object):
         self.items = items
 
     def __eq__(self, other):
-        values, offset = loads(self.argsig, other)
+        values, offset = loads(self.argsig, other, 0)
         return tuple(values) == tuple(self.items)
 
 
