@@ -1,8 +1,9 @@
-from __future__ import absolute_import, unicode_literals
-
 import socket
+from struct import pack
+from unittest.mock import ANY, MagicMock, Mock, patch
 
 import pytest
+from case import ContextMock
 from vine import promise
 
 from amqp import spec
@@ -10,9 +11,7 @@ from amqp.basic_message import Message
 from amqp.channel import Channel
 from amqp.exceptions import (ConsumerCancelled, MessageNacked, NotFound,
                              RecoverableConnectionError)
-from amqp.platform import pack
 from amqp.serialization import dumps
-from case import ANY, ContextMock, MagicMock, Mock, patch
 
 
 class test_Channel:
@@ -404,7 +403,9 @@ class test_Channel:
         self.c._basic_publish.assert_called_with(1, 2, arg=1)
         assert ret is self.c._basic_publish()
         self.c.wait.assert_called_with(
-            [spec.Basic.Ack, spec.Basic.Nack], callback=ANY
+            [spec.Basic.Ack, spec.Basic.Nack],
+            callback=ANY,
+            timeout=None
         )
         self.c.basic_publish_confirm(1, 2, arg=1)
 
