@@ -1,16 +1,15 @@
-from __future__ import absolute_import, unicode_literals
-
+import re
 import socket
 import warnings
+from unittest.mock import Mock, call, patch
 
 import pytest
-from case import ContextMock, Mock, call, patch
+from case import ContextMock
 
 from amqp import Connection, spec
 from amqp.connection import SSLError
 from amqp.exceptions import (ConnectionError, NotFound,
                              RecoverableConnectionError, ResourceError)
-from amqp.five import items
 from amqp.sasl import AMQPLAIN, EXTERNAL, GSSAPI, PLAIN, SASL
 from amqp.transport import TCPTransport
 
@@ -318,7 +317,7 @@ class test_Connection:
         transport = self.conn.transport
         self.conn.collect()
         transport.close.assert_called_with()
-        for i, channel in items(channels):
+        for i, channel in channels.items():
             if i:
                 channel.collect.assert_called_with()
         assert self.conn._transport is None
@@ -503,3 +502,35 @@ class test_Connection:
     def test_server_capabilities(self):
         self.conn.server_properties['capabilities'] = {'foo': 1}
         assert self.conn.server_capabilities == {'foo': 1}
+
+    @pytest.mark.parametrize(
+        'conn_kwargs,expected_vhost', [
+            ({}, '/'),
+            ({'user_id': 'test_user', 'password': 'test_pass'}, '/'),
+            ({'virtual_host': 'test_vhost'}, 'test_vhost')
+        ]
+    )
+    def test_repr_disconnected(self, conn_kwargs, expected_vhost):
+        assert re.fullmatch(
+            r'<AMQP Connection: broker.com:1234/{} '
+            r'\(disconnected\) at 0x.*>'.format(expected_vhost),
+            repr(Connection(host='broker.com:1234', **conn_kwargs))
+        )
+
+    @pytest.mark.parametrize(
+        'conn_kwargs,expected_vhost', [
+            ({}, '/'),
+            ({'user_id': 'test_user', 'password': 'test_pass'}, '/'),
+            ({'virtual_host': 'test_vhost'}, 'test_vhost')
+        ]
+    )
+    def test_repr_connected(self, conn_kwargs, expected_vhost):
+        c = Connection(host='broker.com:1234', **conn_kwargs)
+        c._transport = Mock(name='transport')
+        assert re.fullmatch(
+            r'<AMQP Connection: broker.com:1234/{} using {} at 0x.*>'.format(
+                expected_vhost,
+                repr(c.transport)
+            ),
+            repr(c)
+        )
