@@ -1,17 +1,16 @@
-from __future__ import absolute_import, unicode_literals
-
 import errno
+import os
+import re
 import socket
 import struct
-import os
+from struct import pack
+from unittest.mock import ANY, MagicMock, Mock, call, patch, sentinel
 
 import pytest
 
 from amqp import transport
 from amqp.exceptions import UnexpectedFrame
-from amqp.platform import pack
 from amqp.transport import _AbstractTransport
-from case import ANY, MagicMock, Mock, call, patch
 
 SIGNED_INT_MAX = 0x7FFFFFFF
 
@@ -20,20 +19,22 @@ class DummyException(Exception):
     pass
 
 
-class MockSocket(object):
+class MockSocket:
     options = {}
 
     def __init__(self, *args, **kwargs):
-        super(MockSocket, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connected = False
         self.sa = None
 
     def setsockopt(self, family, key, value):
-        if (family == socket.SOL_SOCKET and
-                key in (socket.SO_RCVTIMEO, socket.SO_SNDTIMEO)):
+        is_sol_socket = family == socket.SOL_SOCKET
+        is_receive_or_send_timeout = key in (socket.SO_RCVTIMEO,
+                                             socket.SO_SNDTIMEO)
+        if is_sol_socket and is_receive_or_send_timeout:
             self.options[key] = value
         elif not isinstance(value, int):
-            raise socket.error()
+            raise OSError()
         self.options[key] = value
 
     def getsockopt(self, family, key):
@@ -53,6 +54,12 @@ class MockSocket(object):
         self.connected = False
         self.sa = None
 
+    def getsockname(self):
+        return ('127.0.0.1', 1234)
+
+    def getpeername(self):
+        return ('1.2.3.4', 5671)
+
 
 TCP_KEEPIDLE = 4
 TCP_KEEPINTVL = 5
@@ -66,12 +73,6 @@ class test_socket_options:
         self.host = '127.0.0.1'
         self.connect_timeout = 3
         self.socket = MockSocket()
-        try:
-            import fcntl
-        except ImportError:
-            fcntl = None
-        if fcntl is not None:
-            patching('fcntl.fcntl')
         socket = patching('socket.socket')
         socket().getsockopt = self.socket.getsockopt
         socket().setsockopt = self.socket.setsockopt
@@ -96,6 +97,7 @@ class test_socket_options:
         patching('amqp.transport.TCPTransport._setup_transport')
         patching('amqp.transport.SSLTransport._write')
         patching('amqp.transport.SSLTransport._setup_transport')
+        patching('amqp.transport.set_cloexec')
 
     def test_backward_compatibility_tcp_transport(self):
         self.transp = transport.Transport(
@@ -179,7 +181,7 @@ class test_socket_options:
             self.transp.connect()
 
     def test_passing_wrong_value_options(self):
-        socket_settings = {TCP_KEEPINTVL: 'a'.encode()}
+        socket_settings = {TCP_KEEPINTVL: b'a'}
         self.transp = transport.Transport(
             self.host, self.connect_timeout,
             socket_settings=socket_settings,
@@ -188,7 +190,7 @@ class test_socket_options:
             self.transp.connect()
 
     def test_passing_value_as_string(self):
-        socket_settings = {TCP_KEEPIDLE: '5'.encode()}
+        socket_settings = {TCP_KEEPIDLE: b'5'}
         self.transp = transport.Transport(
             self.host, self.connect_timeout,
             socket_settings=socket_settings,
@@ -221,8 +223,8 @@ class test_socket_options:
         read_timeout_sec, read_timeout_usec = 0xdead, 0xbeef
         write_timeout_sec = 0x42
 
-        self.transp.read_timeout = read_timeout_sec + \
-            read_timeout_usec * 0.000001
+        read_timeout = read_timeout_sec + read_timeout_usec * 0.000001
+        self.transp.read_timeout = read_timeout
         self.transp.write_timeout = write_timeout_sec
         self.transp.connect()
 
@@ -236,7 +238,6 @@ class test_socket_options:
 
 
 class test_AbstractTransport:
-
     class Transport(transport._AbstractTransport):
 
         def _connect(self, *args):
@@ -246,10 +247,12 @@ class test_AbstractTransport:
             pass
 
     @pytest.fixture(autouse=True)
-    def setup_transport(self):
+    def setup_transport(self, patching):
         self.t = self.Transport('localhost:5672', 10)
         self.t.connect()
 
+        patching('amqp.transport.set_cloexec')
+
     def test_port(self):
         assert self.Transport('localhost').port == 5672
         assert self.Transport('localhost:5672').port == 5672
@@ -322,6 +325,7 @@ class test_AbstractTransport:
             self.t._read.return_value = b'thequickbrownfox'
             self.t._read.side_effect = on_read2
             return ret
+
         self.t._read.return_value = pack('>BHI', 1, 1, 16)
         self.t._read.side_effect = on_read1
 
@@ -329,8 +333,11 @@ class test_AbstractTransport:
         self.t._read.return_value = pack('>BHI', 1, 1, 16)
         self.t._read.side_effect = on_read1
         checksum[0] = b'\x13'
-        with pytest.raises(UnexpectedFrame):
+        with pytest.raises(UnexpectedFrame) as ex:
             self.t.read_frame()
+        assert ex.value.code == 505
+        assert ex.value.message == \
+            'Received frame_end 0x13 while expecting 0xce'
 
     def test_read_frame__long(self):
         self.t._read = Mock()
@@ -342,9 +349,9 @@ class test_AbstractTransport:
         assert payload == b'read1read2'
 
     def transport_read_EOF(self):
-        for host, ssl in (('localhost:5672', False),
-                          ('localhost:5671', True),):
-            self.t = transport.Transport(host, ssl)
+        for host, is_ssl in (('localhost:5672', False),
+                             ('localhost:5671', True),):
+            self.t = transport.Transport(host, is_ssl)
             self.t.sock = Mock(name='socket')
             self.t.connected = True
             self.t._quick_recv = Mock(name='recv', return_value='')
@@ -390,7 +397,7 @@ class test_AbstractTransport:
             sock_mock.gettimeout.return_value = 3
             with self.t.having_timeout(5) as actual_sock:
                 assert actual_sock == self.t.sock
-            sock_mock.gettimeout.assert_called()
+            sock_mock.gettimeout.assert_called_with()
             sock_mock.settimeout.assert_has_calls(
                 [
                     call(5),
@@ -407,7 +414,7 @@ class test_AbstractTransport:
                 with self.t.having_timeout(5) as actual_sock:
                     assert actual_sock == self.t.sock
                     raise DummyException()
-            sock_mock.gettimeout.assert_called()
+            sock_mock.gettimeout.assert_called_with()
             sock_mock.settimeout.assert_has_calls(
                 [
                     call(5),
@@ -422,7 +429,7 @@ class test_AbstractTransport:
             sock_mock.gettimeout.return_value = 5
             with self.t.having_timeout(5) as actual_sock:
                 assert actual_sock == self.t.sock
-            sock_mock.gettimeout.assert_called()
+            sock_mock.gettimeout.assert_called_with()
             sock_mock.settimeout.assert_not_called()
 
     def test_set_timeout_ewouldblock_exc(self):
@@ -445,7 +452,6 @@ class test_AbstractTransport:
 
 
 class test_AbstractTransport_connect:
-
     class Transport(transport._AbstractTransport):
 
         def _init_socket(self, *args):
@@ -454,12 +460,7 @@ class test_AbstractTransport_connect:
     @pytest.fixture(autouse=True)
     def setup_transport(self, patching):
         self.t = self.Transport('localhost:5672', 10)
-        try:
-            import fcntl
-        except ImportError:
-            fcntl = None
-        if fcntl is not None:
-            patching('fcntl.fcntl')
+        patching('amqp.transport.set_cloexec')
 
     def test_connect_socket_fails(self):
         with patch('socket.socket', side_effect=socket.error):
@@ -472,9 +473,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   return_value=[
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672)),
+                       '', ('127.0.0.1', 5672)),
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.2', 5672))
+                       '', ('127.0.0.2', 5672))
                   ]):
             with pytest.raises(socket.error):
                 self.t.connect()
@@ -485,9 +486,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   return_value=[
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672)),
+                       '', ('127.0.0.1', 5672)),
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.2', 5672))
+                       '', ('127.0.0.2', 5672))
                   ]):
             self.t.sock = Mock()
             self.t.close()
@@ -501,9 +502,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   return_value=[
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672)),
+                       '', ('127.0.0.1', 5672)),
                       (socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.2', 5672))
+                       '', ('127.0.0.2', 5672))
                   ]):
             self.t.sock = Mock()
             self.t.close()
@@ -516,9 +517,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   side_effect=[
                       [(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672))],
+                        '', ('127.0.0.1', 5672))],
                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                          '', ('::1', 5672))]
+                        '', ('::1', 5672))]
                   ]) as getaddrinfo:
             self.t.sock = Mock()
             self.t.close()
@@ -531,9 +532,9 @@ class test_AbstractTransport_connect:
             patch('socket.getaddrinfo',
                   side_effect=[
                       [(socket.AF_INET, 1, socket.IPPROTO_TCP,
-                          '', ('127.0.0.1', 5672))],
+                        '', ('127.0.0.1', 5672))],
                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                          '', ('::1', 5672))]
+                        '', ('::1', 5672))]
                   ]) as getaddrinfo:
             self.t.sock = Mock()
             self.t.close()
@@ -555,7 +556,7 @@ class test_AbstractTransport_connect:
                   side_effect=[
                       socket.gaierror,
                       [(socket.AF_INET6, 1, socket.IPPROTO_TCP,
-                          '', ('::1', 5672))]
+                        '', ('::1', 5672))]
                   ]):
             self.t.connect()
 
@@ -580,7 +581,6 @@ class test_AbstractTransport_connect:
 
 
 class test_SSLTransport:
-
     class Transport(transport.SSLTransport):
 
         def _connect(self, *args):
@@ -589,6 +589,20 @@ class test_SSLTransport:
         def _init_socket(self, *args):
             pass
 
+    def test_repr_disconnected(self):
+        assert re.fullmatch(
+            r'<SSLTransport: \(disconnected\) at 0x.*>',
+            repr(transport.SSLTransport('host', 3))
+        )
+
+    def test_repr_connected(self):
+        t = transport.SSLTransport('host', 3)
+        t.sock = MockSocket()
+        re.fullmatch(
+            '<SSLTransport: 127.0.0.1:1234 -> 1.2.3.4:5671 at 0x.*>',
+            repr(t)
+        )
+
     @pytest.fixture(autouse=True)
     def setup_transport(self):
         self.t = self.Transport(
@@ -614,8 +628,8 @@ class test_SSLTransport:
         self.t._wrap_context.assert_called_with(sock, {'foo': 1}, c=2)
 
     def test_wrap_context(self):
-        with patch('ssl.create_default_context', create=True) \
-                as create_default_context:
+        with patch('ssl.create_default_context',
+                   create=True) as create_default_context:
             sock = Mock()
             self.t._wrap_context(sock, {'f': 1}, check_hostname=True, bar=3)
             create_default_context.assert_called_with(bar=3)
@@ -624,15 +638,113 @@ class test_SSLTransport:
             ctx.wrap_socket.assert_called_with(sock, f=1)
 
     def test_wrap_socket_sni(self):
+        # testing default values of _wrap_socket_sni()
         sock = Mock()
-        with patch('ssl.wrap_socket') as mock_ssl_wrap:
-            self.t._wrap_socket_sni(sock)
-            mock_ssl_wrap.assert_called_with(cert_reqs=0, certfile=None,
-                                             keyfile=None, sock=sock,
-                                             ca_certs=None, server_side=False,
-                                             ciphers=None, ssl_version=2,
-                                             suppress_ragged_eofs=True,
-                                             do_handshake_on_connect=False)
+        with patch('ssl.SSLContext') as mock_ssl_context_class:
+            wrap_socket_method_mock = mock_ssl_context_class().wrap_socket
+            wrap_socket_method_mock.return_value = sentinel.WRAPPED_SOCKET
+            ret = self.t._wrap_socket_sni(sock)
+
+        mock_ssl_context_class.load_cert_chain.assert_not_called()
+        mock_ssl_context_class.load_verify_locations.assert_not_called()
+        mock_ssl_context_class.set_ciphers.assert_not_called()
+        mock_ssl_context_class.verify_mode.assert_not_called()
+        wrap_socket_method_mock.assert_called_with(
+            sock=sock,
+            server_side=False,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=True,
+            server_hostname=None
+        )
+        assert ret == sentinel.WRAPPED_SOCKET
+
+    def test_wrap_socket_sni_certfile(self):
+        # testing _wrap_socket_sni() with parameters certfile and keyfile
+        with patch('ssl.SSLContext') as mock_ssl_context_class:
+            load_cert_chain_method_mock = \
+                mock_ssl_context_class().load_cert_chain
+            self.t._wrap_socket_sni(
+                Mock(), keyfile=sentinel.KEYFILE, certfile=sentinel.CERTFILE
+            )
+
+        load_cert_chain_method_mock.assert_called_with(
+            sentinel.CERTFILE, sentinel.KEYFILE
+        )
+
+    def test_wrap_socket_ca_certs(self):
+        # testing _wrap_socket_sni() with parameter ca_certs
+        with patch('ssl.SSLContext') as mock_ssl_context_class:
+            load_verify_locations_method_mock = \
+                mock_ssl_context_class().load_verify_locations
+            self.t._wrap_socket_sni(Mock(), ca_certs=sentinel.CA_CERTS)
+
+        load_verify_locations_method_mock.assert_called_with(sentinel.CA_CERTS)
+
+    def test_wrap_socket_ciphers(self):
+        # testing _wrap_socket_sni() with parameter ciphers
+        with patch('ssl.SSLContext') as mock_ssl_context_class:
+            set_ciphers_method_mock = mock_ssl_context_class().set_ciphers
+            self.t._wrap_socket_sni(Mock(), ciphers=sentinel.CIPHERS)
+
+        set_ciphers_method_mock.assert_called_with(sentinel.CIPHERS)
+
+    def test_wrap_socket_sni_cert_reqs(self):
+        # testing _wrap_socket_sni() with parameter cert_reqs
+        with patch('ssl.SSLContext') as mock_ssl_context_class:
+            self.t._wrap_socket_sni(Mock(), cert_reqs=sentinel.CERT_REQS)
+
+        assert mock_ssl_context_class().verify_mode == sentinel.CERT_REQS
+
+    def test_wrap_socket_sni_setting_sni_header(self):
+        # testing _wrap_socket_sni() without parameter server_hostname
+        # SSL module supports SNI
+        with patch('ssl.SSLContext') as mock_ssl_context_class, \
+                patch('ssl.HAS_SNI', new=True):
+            self.t._wrap_socket_sni(Mock())
+
+        assert mock_ssl_context_class().check_hostname is False
+
+        # SSL module does not support SNI
+        with patch('ssl.SSLContext') as mock_ssl_context_class, \
+                patch('ssl.HAS_SNI', new=False):
+            self.t._wrap_socket_sni(Mock())
+
+        assert mock_ssl_context_class().check_hostname is False
+
+        # testing _wrap_socket_sni() with parameter server_hostname
+        sock = Mock()
+        with patch('ssl.SSLContext') as mock_ssl_context_class, \
+                patch('ssl.HAS_SNI', new=True):
+            # SSL module supports SNI
+            wrap_socket_method_mock = mock_ssl_context_class().wrap_socket
+            self.t._wrap_socket_sni(
+                sock, server_hostname=sentinel.SERVER_HOSTNAME
+            )
+
+        wrap_socket_method_mock.assert_called_with(
+            sock=sock,
+            server_side=False,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=True,
+            server_hostname=sentinel.SERVER_HOSTNAME
+        )
+        assert mock_ssl_context_class().check_hostname is True
+
+        with patch('ssl.SSLContext') as mock_ssl_context_class, \
+                patch('ssl.HAS_SNI', new=False):
+            # SSL module does not support SNI
+            wrap_socket_method_mock = mock_ssl_context_class().wrap_socket
+            self.t._wrap_socket_sni(
+                sock, server_hostname=sentinel.SERVER_HOSTNAME
+            )
+        wrap_socket_method_mock.assert_called_with(
+            sock=sock,
+            server_side=False,
+            do_handshake_on_connect=False,
+            suppress_ragged_eofs=True,
+            server_hostname=sentinel.SERVER_HOSTNAME
+        )
+        assert mock_ssl_context_class().check_hostname is False
 
     def test_shutdown_transport(self):
         self.t.sock = None
@@ -654,7 +766,7 @@ class test_SSLTransport:
         self.t.sock = Mock(name='SSLSocket')
         self.t.sock.write.return_value = 2
         self.t._write('foo')
-        self.t.sock.write.assert_called()
+        self.t.sock.write.assert_called_with(ANY)
 
     def test_write_socket_closed(self):
         self.t.sock = Mock(name='SSLSocket')
@@ -682,14 +794,13 @@ class test_SSLTransport:
     def test_read_SSLError(self):
         self.t.sock = Mock(name='SSLSocket')
         self.t._quick_recv = Mock(name='recv', return_value='4')
-        self.t._quick_recv.side_effect = transport.SSLError('timed out')
+        self.t._quick_recv.side_effect = socket.timeout()
         self.t._read_buffer = MagicMock(return_value='AA')
         with pytest.raises(socket.timeout):
             self.t._read(64)
 
 
 class test_TCPTransport:
-
     class Transport(transport.TCPTransport):
 
         def _connect(self, *args):
@@ -698,6 +809,20 @@ class test_TCPTransport:
         def _init_socket(self, *args):
             pass
 
+    def test_repr_disconnected(self):
+        assert re.fullmatch(
+            r'<TCPTransport: \(disconnected\) at 0x.*>',
+            repr(transport.TCPTransport('host', 3))
+        )
+
+    def test_repr_connected(self):
+        t = transport.SSLTransport('host', 3)
+        t.sock = MockSocket()
+        re.fullmatch(
+            '<TCPTransport: 127.0.0.1:1234 -> 1.2.3.4:5671 at 0x.*>',
+            repr(t)
+        )
+
     @pytest.fixture(autouse=True)
     def setup_transport(self):
         self.t = self.Transport('host', 3)
